﻿#region Copyright (c) 2016-2020 Alternet Software

/*
    AlterNET Code Editor Library

    Copyright (c) 2016-2020 Alternet Software
    ALL RIGHTS RESERVED

    http://www.alternetsoft.com
    contact@alternetsoft.com
*/

#endregion Copyright (c) 2016-2020 Alternet Software

using System;
using System.IO;
using System.Windows.Forms;
using System.Xml.Schema;

using Alternet.Syntax;
using Alternet.Syntax.Lexer;
using NJsonSchema;

namespace XMLJSONValidator
{
    public partial class Form1 : Form
    {
        private const string LanguageDescription = "Choose programming language";

        private string dir = Application.StartupPath + @"\";

        public Form1()
        {
            InitializeComponent();
        }

        private static ILexer CreateXmlParser()
        {
            var result = new ValidatingXmlParser();
            const string SchemaFileName = @"..\..\Schemas\XmlSchema.xsd";
            if (File.Exists(SchemaFileName))
            {
                using (var file = File.OpenRead(SchemaFileName))
                    result.Schema = XmlSchema.Read(file, null);
            }

            return result;
        }

        private static ILexer CreateJsonParser()
        {
            var result = new ValidatingJsonParser();
            const string SchemaFileName = @"..\..\Schemas\JsonSchema.schema.json";
            if (File.Exists(SchemaFileName))
                result.Schema = JsonSchema.FromFileAsync(SchemaFileName).Result;

            return result;
        }

        private string GetFileName(int index)
        {
            string path = Path.Combine(dir, @"..\..\Text");
            switch (index)
            {
                case 0:
                    return Path.Combine(path, "json.txt");

                case 1:
                    return Path.Combine(path, "xml.txt");

                default:
                    throw new InvalidOperationException();
            }
        }

        private ILexer GetLexer(int index)
        {
            ILexer CreateLexer()
            {
                switch (index)
                {
                    case 0:
                        return CreateJsonParser();

                    case 1:
                        return CreateXmlParser();

                    default:
                        throw new InvalidOperationException();
                }
            }

            var lexer = CreateLexer();
            if (lexer is ISyntaxParser syntaxParser)
                syntaxParser.Options |= SyntaxOptions.CodeCompletion | SyntaxOptions.QuickInfoTips | SyntaxOptions.SyntaxErrors;

            return lexer;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            cbLanguages.SelectedIndex = 1;
        }

        private void cbLanguages_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cbLanguages.SelectedIndex >= 0)
            {
                syntaxEdit1.Source.Lexer = GetLexer(cbLanguages.SelectedIndex);
                string fileName = GetFileName(cbLanguages.SelectedIndex);
                if (fileName != string.Empty)
                    syntaxEdit1.Source.LoadFile(fileName);
            }
        }

        private void cbLanguages_MouseMove(object sender, MouseEventArgs e)
        {
            string str = toolTip1.GetToolTip(cbLanguages);
            if (str != LanguageDescription)
                toolTip1.SetToolTip(cbLanguages, LanguageDescription);
        }
    }
}